unit LCD; {Original von abacom erweitert}

interface

uses CH341DLL, Windows, SysUtils;


// ***************************************************************
// LCD Text Display (HD44780 controller or compatible) with CH341A
//
// API: MemWriteAddr0, MemWriteAddr1
//
// Connections:
//
// LCD pin                   CH341A pin    comment
// ---------------------------------------------------------------
// D0...D7                    D0..D7   8 bit interface
// E                          /WR      clock inverted; no problem!
// RS                         A0       0=control reg. 1=data reg.
// R/W                        Ground   always write; never read
// ***************************************************************

Const

   //HD44780 instructions, compare HD44780 data sheet

   cmd_Clear_Display            = $01;
   cmd_Return_Home              = $02;
   cmd_Entry_Mode_Set           = $04;
   cmd_Display_On_Off           = $08;
   cmd_Cursor_or_Display_Shift  = $10;
   cmd_Function_Set             = $20;
   cmd_Set_CGRAM_Address        = $40;
   cmd_Set_DDRAM_Address        = $80;

   //HD44780 instruction params

   // cmd_Entry_Mode_Set params
   cursor_Shift_Left            =$00;
   cursor_Shift_Right           =$02;
   cursor_Shift_On              =$01;
   cursor_Shift_Off             =$00;

   // cmd_Display_On_Off params
   display_On                   =$04;
   display_Off                  =$00;
   cursor_On                    =$02;
   cursor_Off                   =$00;
   Cursor_Blink_On              =$01;
   Cursor_Blink_Off             =$00;

   // cmd_Cursor_or_Display_Shift params
   Shift_Display                =$08;
   Shift_Cursor                 =$00;
   Shift_Left                   =$04;
   Shift_Right                  =$00;

   //cmd_Function_Set params
   interface_8Bit               =$10;
   interface_4Bit               =$00;
   display_2lines               =$08;
   display_1line                =$00;
   Font_5x11                    =$04;
   Font_5x8                     =$00;

type TDisplayFormat = (lcd1x8, lcd1x16, lcd88, lcd1x20, lcd1x40,
                       lcd2x8, lcd2x12, lcd2x16,lcd2x20, lcd2x24, lcd2x40,
                       lcd4x16, lcd4x20);

    procedure LCDInit(Device: Integer);
    procedure WriteToLCDControlReg(Device: Integer; aByte: Byte);
    procedure WriteToLCDDataReg(Device: Integer; Data: String);
    procedure SetDDRAMaddress(Device: Integer; Adr: Byte);
    procedure SetCGRAMaddress(Device: Integer; Adr: Byte);
    procedure WriteStringToDDRAM(Device: Integer; aString: String; DDRAMaddress: Byte);
    function RowStartAddress(aRow: Byte; aDisplayFormat: TDisplayFormat): Byte;
    function NumRows(aDisplayFormat: TDisplayFormat): Byte;
    function NumColumns(aDisplayFormat: TDisplayFormat): Byte;
    procedure WriteStringToRow(Device: Integer; aString: String; aRow: Byte; aDisplayFormat: TDisplayFormat);
    procedure UpdateRow(Device: Integer; New, Old: String; aRow: Byte; aDisplayFormat: TDisplayFormat);
    function CheckStringLength(aString: String; aLen: Integer): String;

implementation

procedure WriteToLCDControlReg(Device: Integer; aByte: Byte);
var BufferLen: cardinal;
begin
   BufferLen:=1;
   // Write with A0=0 selects LCD control reg.
   CH341MemWriteAddr0(Device, @aByte, @BufferLen); //Adr. 0 = Control Reg.
   sleep(1);
end;

procedure WriteToLCDDataReg(Device: Integer; Data: String);
var BufferLen: cardinal;
    i: integer;
    aChar: Char;
begin
   BufferLen:=1;
   For i:=1 to Length(Data) do //write one by one; LCD needs delays!
   begin
      aChar:=Data[i];
      // Write with A0=1 selects LCD data reg.
      CH341MemWriteAddr1(Device, @aChar, @BufferLen); //Adr. 1 = Data Reg.
   end;
end;

procedure LCDInit(Device: Integer);
var i: Integer;
begin
   WriteToLCDControlReg(Device,cmd_Function_Set or interface_8Bit);
   sleep(10);
   For i:=1 to 3 do
   begin
      WriteToLCDControlReg(Device, cmd_Function_Set or interface_8Bit);
   end;
   WriteToLCDControlReg(Device, cmd_Function_Set or Interface_8Bit or display_2lines or Font_5x11);
   WriteToLCDControlReg(Device, cmd_Display_On_Off or display_off or cursor_off or cursor_blink_off);
   WriteToLCDControlReg(Device, cmd_Clear_Display);
   WriteToLCDControlReg(Device, cmd_Entry_Mode_Set or cursor_Shift_Off or Cursor_Shift_right);
   WriteToLCDControlReg(Device, cmd_Display_On_Off or display_On or cursor_off or cursor_blink_off);
end;

procedure SetDDRAMaddress(Device: Integer; Adr: Byte);
begin
   WriteToLCDControlReg(Device, cmd_Set_DDRAM_Address or (Adr and $7F));
   Sleep(1);
end;

procedure SetCGRAMaddress(Device: Integer; Adr: Byte);
begin
   WriteToLCDControlReg(Device, cmd_Set_CGRAM_Address or (Adr and $3F));
   Sleep(1);
end;

//String ins RAM schreiben
procedure WriteStringToDDRAM(Device: Integer; aString: String; DDRAMaddress: Byte);
begin
   aString:=copy(aString,1,$7F);
   SetDDRAMaddress(Device, DDRAMaddress);
   WriteToLCDDataReg(Device, aString);
end;

// String in eine Zeile schreiben
procedure WriteStringToRow(Device: Integer; aString: String; aRow: Byte; aDisplayFormat: TDisplayFormat);
begin
   aString:=CheckStringLength(aString, NumColumns(aDisplayFormat));
   WriteStringToDDRAM(Device, aString, RowStartAddress(aRow, aDisplayFormat));
end;


// Nur Vernderungen in einer LCD-Zeile werden an das LCD bertragen...
procedure UpdateRow(Device: Integer; New, Old: String; aRow: Byte; aDisplayFormat: TDisplayFormat);
var changes: String;
    i, Adr: Integer;
begin
   // Stringlngen an das Display anpassen...
   New:=CheckStringLength(New, NumColumns(aDisplayFormat));
   Old:=CheckStringLength(Old, NumColumns(aDisplayFormat));

   //Gesammte Zeile spaltenweise durchlaufen...
   i:=1;
   while (i<=NumColumns(aDisplayFormat)) do
   begin
      Adr:=i-1;
      //Solange Zeichen verndert sind: Merken bis keine Vernderung mehr
      changes:='';
      while (new[i]<>old[i])and (i<=NumColumns(aDisplayFormat)) do
      begin
         changes:=changes+new[i];
         inc(i);
      end;
      // Die Vernderung ausgeben...
      if changes<>'' then WriteStringToDDRam(Device, Changes, RowStartAddress(aRow, aDisplayFormat)+Adr);
      // Solang keine Vernderung vorhanden ist: Nichts machen...
      while (new[i]=old[i])and (i<=NumColumns(aDisplayFormat)) do
      begin
         inc(i);
      end;
   end; //Bis Zeile ganz durchlaufen ist
end;

//String auf bestimmte lnge bringen
function CheckStringLength(aString: String; aLen: Integer): String;
begin
   //Zu lange Strings krzen
   result:=Copy(aString,1,aLen);
   //Zu kurze Strings mit Leerzeichen auffllen
   While length(result)<aLen do result:=result+' ';
end;

//Anzahl der Zeilen je nach Displayformat
function NumRows(aDisplayFormat: TDisplayFormat): Byte;
begin
   case aDisplayFormat of
   lcd1x8, lcd1x16, lcd1x20, lcd1x40: result:=1;
   lcd88, lcd2x8, lcd2x12, lcd2x16,lcd2x20, lcd2x24, lcd2x40: result:=2;
   lcd4x16, lcd4x20: result:=4;
   else result:=0;
   end;
end;

// Anzahl der Spalten je nach Displayformat
function NumColumns(aDisplayFormat: TDisplayFormat): Byte;
begin
   case aDisplayFormat of
   lcd1x8, lcd88, lcd2x8: result:=8;
   lcd2x12: result:=12;
   lcd1x16, lcd2x16, lcd4x16: result:=16;
   lcd1x20, lcd2x20, lcd4x20: result:=20;
   lcd1x40, lcd2x40: result:=40;
   lcd2x24: result:=24;
   else result:=0;
   end;
end;

// RAM Startadresse einer Zeile je nach Displayformat
function RowStartAddress(aRow: Byte; aDisplayFormat: TDisplayFormat): Byte;
begin
   case aRow of
   //Erste Zeile immer an Adr. $00
   1: result:=$00;
   //Zeite Zeile immer an Adr. $40
   2: result:=$40;
   //Dritte Zeile, wenn vorhanden...
   3: case aDisplayFormat of
      LCD4x16: result:=$10;
      LCD4x20: result:=$14;
      else result:=0;
      end;
   //Vierte Zeile, wenn vorhanden...
   4: case aDisplayFormat of
      LCD4x16: result:=$50;
      LCD4x20: result:=$54;
      else result:=0;
      end;
   else result:=0;
   end;
end;

end.
