////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// LCDHype LCD Driver DLL Source Code                                         //
//                                                                            //
// Version 1.2                                                                //
//                                                                            //
// Requirements: LCDHype Version 0.37 or above                                //
//               DriverLINX DLPortIO Driver installed                         //
//               (if not needed remove the imports)                           //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

unit controller;

interface

uses ch341dll, lcd, sysUtils;

const ID = 'CH341A USB LCD by ExpertProfi'; //ID sets the description of the dll
      MAX_RESPIXELS = 76800;

var LCD_Format: TDisplayFormat;
    CH341ADeviceNo : Integer;

type TCharData = record
                  position : byte;
                  data : array[0..7,0..7] of byte;
                 end;
     TDLLInfo = record
                 IDArray : array[0..255] of byte; //holds identifier of DLL
                 SupportGfxLCD : Boolean;         //does this driver support graphical LCDs?
                 SupportTxtLCD : Boolean;         //does this driver support text
                 SupportLightSlider : Boolean;    //does this driver support the light control slider
                 SupportContrastSlider : Boolean; //does this driver support the contrast slider
                 SupportOutports : Boolean;       //does this driver support outports for controlling external circuits
                 CCharWidth : Byte;               //custom char width in pixels
                 CCharHeight : Byte;              //custom char height in pixels
                 FontPitch : Byte;                //fontpitch of LCD in pixels
                end;
     TSymbolInfo = record
                    SymbolData : array[0..19] of boolean;
                   end;

     TArrayType = array[0..MAX_RESPIXELS-1] of byte;

//Sends Data to the Controller Chip
procedure LCD_SendToController(Value : Byte); stdcall;

//Sends Data to the integrated LCD Memory
//on adress specified by LCD_SetOutputAdress
procedure LCD_SendToMemory(Value : Byte); stdcall;

//Sends Data to LCD Gfx Memory
procedure LCD_SendToGfxMemory(Pixels : TArrayType; X1,Y1,X2,Y2 : Word; inverted : boolean); stdcall;

//Sets the LCD RAM Adress for Outputs
procedure LCD_SetOutputAddress(x,y : Longword); stdcall;

//Sets a LCD Custom Char specified in cdata
procedure LCD_SetCGRAMChar(cdata : TCharData); stdcall;

//Returns the Custom Char bytecode for the current display
//Based on the HD44780 eg. the input for the first Custom Char
//is position=0 and the return value is the equivalent bytecode
//to the HD44780 Custom Char bytecode
function LCD_GetCGRAMChar(position : Byte) : Byte; stdcall;

//Initialize the LCD
procedure LCD_Init; stdcall;

//Called when DLL released
procedure LCD_CleanUp; stdcall;

//Sets the Port Adress where the LCD are connected and
//the lcd execution time for processing sended data
procedure LCD_SetIOPropertys(Port : pchar;
                             Exectime : longword;
                             ExectimeGfx : longword;
                             X : longword;
                             Y : longword;
                             gX : longword;
                             gY : longword;
                             LightOn : Boolean;
                             LightSliderValue : Byte;
                             ContrastOn : Boolean;
                             ContrastSliderValue : Byte;
                             Outports : Longword;
                             UnderlineMode : Boolean;
                             UnderlineOutput : Boolean); stdcall;

//This is a routine for customizing output,
//by default only used by m50530 symbol lcd
procedure LCD_Customize(SymData : TSymbolInfo); stdcall;

//Caller of this function gets some info about the DLL
function DLL_GetInfo : TDLLInfo; stdcall;

//Signals if the write sequence to LCD is ended or not
function LCD_IsReadyToReceive : boolean; stdcall;

//Signals if the output device supports color mode.
//When you set the return value to true LCDHype will pass
//encoded color values (each byte in TArrayType, LCD_SendToGfxMemory).
//This measn that the most significant bits of a byte of the red, green and
//blue color part is encoded in a 3:3:2 ration into one byte.
function LCD_HasColorDisplay : boolean; stdcall;

//Imported DLL Routine from DriverLinx PortIO DLL Driver
//function  DlPortReadPortUchar(Port : Longword) : Byte; stdcall; external 'DLPORTIO.DLL';
//function  DlPortReadPortUchar(Port : Longword) : Byte;

implementation

function LCD_HasColorDisplay : boolean; stdcall;
begin
   result:=false;
end;

procedure LCD_SetIOPropertys(Port : pchar;
                             Exectime : longword;
                             ExectimeGfx : longword;
                             X : longword;
                             Y : longword;
                             gX : longword;
                             gY : longword;
                             LightOn : Boolean;
                             LightSliderValue : Byte;
                             ContrastOn : Boolean;
                             ContrastSliderValue : Byte;
                             Outports : Longword;
                             UnderlineMode : Boolean;
                             UnderlineOutput : Boolean);
begin
   CH341ADeviceNo:=StrToInt(Port);
   case y of
   1:
   begin
      case x of
      8: lcd_Format:=lcd1x8;
      16: lcd_Format:=lcd1x16;
      20: lcd_Format:=lcd1x20;
      40: lcd_Format:=lcd1x40;
      end;
   end;
   2:
   begin
      case x of
      8: lcd_Format:=lcd2x8;
      12: lcd_Format:=lcd2x12;
      16: lcd_Format:=lcd2x16;
      20: lcd_Format:=lcd2x20;
      24: lcd_Format:=lcd2x24;
      40: lcd_Format:=lcd2x40;
      end;
   end;
   4:
   begin
      case x of
      16: lcd_Format:=lcd4x16;
      20: lcd_Format:=lcd4x20;
      end;
   end;
   end;
end;

function DLL_GetInfo : TDLLInfo;
var c : integer; x : TDLLInfo;
begin
     //copy lcd driver name to structure
     for c:=0 to 255 do x.IDArray[c] := 0;
     for c:=1 to length(ID) do begin
      x.IDArray[c-1] := ord(ID[c]);
     end;

     x.SupportGfxLCD := false; //do we support gfx output?
     x.SupportTxtLCD := true;  //do we support txt output?
     x.SupportLightSlider := false; //do we support the slider in lcdhype output?
     x.SupportContrastSlider := false;
     x.SupportOutports := false;
     x.CCharWidth := 0; //max custom char width in pixels
     x.CCharHeight := 0; //max custom char height in pixels
     x.FontPitch := 6; //fontpitch of lcd

     result := x;
end;

//a small routine for generating very short delays
procedure Wait(x : longword);
var c : longword;
begin
     for c:=0 to x do;
//      DlPortReadPortUchar($220); //read = 1 microsecond delay
end;

function LCD_IsReadyToReceive : boolean;
begin
   result:=true;
end;

procedure LCD_SendToController(Value : Byte);
begin
   WriteToLCDControlReg(Ch341ADeviceNo, Value)
end;

procedure LCD_SendToGfxMemory(Pixels : TArrayType; X1,Y1,X2,Y2 : Word; inverted : boolean);
begin
 // nicht untersttzt
end;

procedure LCD_SendToMemory(Value : Byte);
var buffer: Char;
begin
   Buffer:=chr(Value);
   WriteToLCDDataReg(Ch341ADeviceNo, Buffer)
end;

procedure LCD_SetOutputAddress(x,y : Longword);
var Adr: Byte;
begin
   Adr:=RowStartAddress(y+1, LCD_Format) + x;
   SetDDRamAddress(CH341ADeviceNo, Adr);
end;



procedure LCD_SetCGRAMChar(cdata : TCharData);
var buffer: Char;
    aByte: Byte;
    i,ii: integer;
begin
 // Nicht untersttzt
   SetCGRamAddress(CH341ADeviceNo, cData.Position*8);
   For i:=0 to 7 do
   begin
      aByte:=0;
      For ii:=0 to 4 do
      begin
         if cData.data[4-ii,i]<>0 then aByte:=aByte or (1 shl ii)
      end;
      Buffer:=chr(aByte);
      WriteToLCDDataReg(Ch341ADeviceNo, Buffer);
   end;
end;



function LCD_GetCGRAMChar(position : Byte) : Byte;
begin
   result := position;
end;

procedure LCD_Init;
begin
   CH341OpenDevice(CH341ADeviceNo);
   LCDInit(CH341ADeviceNo);
end;

procedure LCD_CleanUp;
begin
   CH341CloseDevice(CH341ADeviceNo);
end;

procedure LCD_Customize(SymData : TSymbolInfo);
begin
   //Nicht untersttzt
end;

begin
end.

